#
# spec file for package rocksdb
#
# Copyright (c) 2023 SUSE LLC
# Copyright (c) 2023 Andreas Stieger <Andreas.Stieger@gmx.de>
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define lib_name librocksdb8
%bcond_with jemalloc
Name:           rocksdb
Version:        8.5.3
Release:        1.1
Summary:        Library for embeddable, persistent and fast key-value store
License:        (Apache-2.0 OR GPL-2.0-only) AND BSD-2-Clause
URL:            https://rocksdb.org/
Source:         https://github.com/facebook/rocksdb/archive/v%{version}.tar.gz#/%{name}-%{version}.tar.gz
Patch0:         rocksdb-8.0.0-reproducible.patch
Patch1:         rocksdb-8.0.0-shared-liburing.patch
Patch2:         rocksdb-8.0.0-rpath.patch
BuildRequires:  c++_compiler
BuildRequires:  cmake
BuildRequires:  pkgconfig
BuildRequires:  cmake(Snappy)
BuildRequires:  pkgconfig(bzip2)
BuildRequires:  pkgconfig(liblz4)
BuildRequires:  pkgconfig(liburing)
BuildRequires:  pkgconfig(libzstd)
BuildRequires:  pkgconfig(zlib)
%if %{with jemalloc}
BuildRequires:  pkgconfig(jemalloc)
%endif
# see SR#1075555 for gflags linking failure
%if 0%{?suse_version} > 1500
BuildRequires:  pkgconfig(gflags)
%else
BuildRequires:  gflags-devel-static
%endif

%description
RocksDB is a high performance embedded database for key-value data.
It is a fork of LevelDB which was then optimized to exploit many
central processing unit (CPU) cores, and make efficient use of fast
storage, such as solid-state drives (SSD), for input/output (I/O)
bound workloads. It is based on a log-structured merge-tree (LSM tree)
data structure.

%package -n %{lib_name}
Summary:        Shared library from rocksdb

%description -n %{lib_name}
RocksDB is a high performance embedded database for key-value data.
It is a fork of LevelDB which was then optimized to exploit many
central processing unit (CPU) cores, and make efficient use of fast
storage, such as solid-state drives (SSD), for input/output (I/O)
bound workloads. It is based on a log-structured merge-tree (LSM tree)
data structure.

This package holds the shared library of rocksdb.

%package tools
Summary:        Utility tools for RocksDB
# MariaDB ships /usr/bin/sst_dump - MDEV-14918
Conflicts:      mariadb

%description tools
RocksDB is a high performance embedded database for key-value data.
This package contains utility tools for RocksDB.

%package devel
Summary:        Development package for RocksDB
Requires:       %{lib_name} = %{version}

%description devel
RocksDB is a high performance embedded database for key-value data.
It is a fork of LevelDB which was then optimized to exploit many
central processing unit (CPU) cores, and make efficient use of fast
storage, such as solid-state drives (SSD), for input/output (I/O)
bound workloads. It is based on a log-structured merge-tree (LSM tree)
data structure.

This package contains the files needed to compile programs that use
the RocksDB library.

%prep
%autosetup -p1

%build
# building tests is disabled, because they require additional instrumentation,
# which is build in library in debug mode and adds some overhead.
# Warnings: https://github.com/facebook/rocksdb/issues/11043
%cmake \
    -DPORTABLE=1 \
    -DFAIL_ON_WARNINGS=OFF \
%if !%{with jemalloc}
    -DWITH_JEMALLOC=0 \
%endif
    -DWITH_SNAPPY=ON \
    -DWITH_LZ4=ON \
    -DWITH_ZLIB=ON \
    -DWITH_ZSTD=ON \
    -DWITH_BZ2=ON \
    -DWITH_TESTS=OFF \
    -DWITH_TOOLS=OFF \
    -DWITH_BENCHMARK_TOOLS=OFF \
    %{nil}

%cmake_build

%install
%cmake_install
find %{buildroot}%{_libdir} -type f -name "*.a" -print -delete
install -dD -m 755 %{buildroot}/%{_bindir}
install -m 755 build/tools/ldb %{buildroot}/%{_bindir}/ldb
install -m 755 build/tools/sst_dump %{buildroot}/%{_bindir}/sst_dump

%ldconfig_scriptlets -n %{lib_name}

%files -n %{lib_name}
%license COPYING LICENSE.Apache LICENSE.leveldb
%{_libdir}/librocksdb.so.*

%files tools
%license COPYING LICENSE.Apache LICENSE.leveldb
%{_bindir}/ldb
%{_bindir}/sst_dump

%files devel
%license COPYING LICENSE.Apache LICENSE.leveldb
%doc README.md HISTORY.md LANGUAGE-BINDINGS.md
%{_includedir}/rocksdb
%{_libdir}/librocksdb.so
%{_libdir}/pkgconfig/rocksdb.pc
%{_libdir}/cmake/rocksdb

%changelog
* Wed Sep  6 2023 Andrea Manzini <andrea.manzini@suse.com>
- update to 8.5.3
  * Fixed a race condition in GenericRateLimiter that could cause it to stop granting requests
- update to 8.5.2
  * Fix a bug where iterator may return incorrect result for DeleteRange() users if there was an error reading from a file.
- update to 8.5.1
  * Fix a bug where if there is an error reading from offset 0 of a file from L1+ and that the file is not the first file in the sorted run, data can be lost in compaction and read/scan can return incorrect results.
- update to 8.5.0
  * Public API Changes:
    Removed recently added APIs GeneralCache and MakeSharedGeneralCache() as our plan changed to stop exposing a general-purpose cache interface. The old forms of these APIs, Cache and NewLRUCache(), are still available, although general-purpose caching support will be dropped eventually.
  * Behavior Changes
    Option periodic_compaction_seconds no longer supports FIFO compaction: setting it has no effect on FIFO compactions. FIFO compaction users should only set option ttl instead.
    Move prefetching responsibility to page cache for compaction read for non directIO use case
  * Performance Improvements
    In case of direct_io, if buffer passed by callee is already aligned, RandomAccessFileRead::Read will avoid realloacting a new buffer, reducing memcpy and use already passed aligned buffer.
    Small efficiency improvement to HyperClockCache by reducing chance of compiler-generated heap allocations
  * Bug Fixes
    Fix use_after_free bug in async_io MultiReads when underlying FS enabled kFSBuffer. kFSBuffer is when underlying FS pass their own buffer instead of using RocksDB scratch in FSReadRequest. Right now it's an experimental feature.
    Fix a bug in FileTTLBooster that can cause users with a large number of levels (more than 65) to see errors like "runtime error: shift exponent .. is too large.."
- see more on HISTORY.md (https://github.com/facebook/rocksdb/blob/main/HISTORY.md)
* Thu Jul 27 2023 Andrea Manzini <andrea.manzini@suse.com>
- update to 8.3.2
  * Bug Fixes:
    Reduced cases of illegally using Env::Default() during static destruction by never destroying the internal PosixEnv itself
- update to 8.3.1
  * Performance Improvements:
    Fixed higher read QPS during DB::Open() reading files created prior to #11406,
- update to 8.3.0
  * New Features:
    Introduced a new option block_protection_bytes_per_key, which can be used to enable per key-value integrity protection for in-memory blocks in block cache
    Improve the operational safety of publishing a DB or SST files to many hosts by using different block cache hash seeds on different hosts.
    Introduced a new option CompactionOptionsFIFO::file_temperature_age_thresholds that allows FIFO compaction to compact files to different temperatures based on key age
    Added a new ticker stat to count how many times RocksDB detected a corruption while verifying a block checksum: BLOCK_CHECKSUM_MISMATCH_COUNT.
    New statistics rocksdb.file.read.db.open.micros that measures read time of block-based SST tables or blob files during db open.
    New statistics tickers for various iterator seek behaviors and relevant filtering, as *_LEVEL_SEEK_*. (#11460)
  * Public API Changes:
    EXPERIMENTAL: Add new API DB::ClipColumnFamily to clip the key in CF to a certain range. It will physically deletes all keys outside the range including tombstones.
    Add MakeSharedCache() construction functions to various cache Options objects, and deprecated the NewWhateverCache() functions with long parameter lists.
    Changed the meaning of various Bloom filter stats (prefix vs. whole key), with iterator-related filtering only being tracked in the new *_LEVEL_SEEK_*. stats. (#11460)
  * Behavior changes:
    For x86, CPU features are no longer detected at runtime nor in build scripts, but in source code using common preprocessor defines.
    This will likely unlock some small performance improvements on some newer hardware, but could hurt performance of the kCRC32c checksum,
    which is no longer the default, on some "portable" builds. See PR #11419 for details.
  * Bug Fixes:
    Delete an empty WAL file on DB open if the log number is less than the min log number to keep
    Delete temp OPTIONS file on DB open if there is a failure to write it out or rename it
  * Performance Improvements:
    Improved the I/O efficiency of prefetching SST metadata by recording more information in the DB manifest.
* Fri Apr 21 2023 Andreas Stieger <Andreas.Stieger@gmx.de>
- RocksDB 8.1.1
  * Compaction output file cutting logic now considers range
    tombstone start keys. For example, SST partitioner now may
    receive ParitionRequest for range tombstone start keys.
  * If the async_io ReadOption is specified for MultiGet or
    NewIterator on a platform that doesn't support IO uring, the
    option is ignored and synchronous IO is used.
  * Add support for SecondaryCache with HyperClockCache
  * New statistics and performance counters
  * bug fixes
  * API changes: Changed various functions and features in Cache
    that are mostly relevant to custom implementations or wrappers
- build with shared gflags libs
* Sat Apr 15 2023 Andreas Stieger <andreas.stieger@gmx.de>
- add hardware constraints for reliable builds
* Sat Mar 25 2023 Andreas Stieger <andreas.stieger@gmx.de>
- Disable usage of jemalloc for gh#jemalloc/jemalloc#1237 to avoid
  "TLS error: cannot allocate memory in static TLS block"
* Wed Mar 22 2023 Andreas Stieger <andreas.stieger@gmx.de>
- RocksDB 8.0.0
- add rocksdb-8.0.0-reproducible.patch - make build reproducible
- add rocksdb-8.0.0-shared-liburing.patch - liburing dynamic linking
- add rocksdb-8.0.0-rpath.patch
* Wed Oct 28 2020 Adam Mizerski <adam@mizerski.pl>
- update to 6.13.3
  * see packaged HISTORY.md
- refreshed patch gtest.patch
* Sat Feb  2 2019 Adam Mizerski <adam@mizerski.pl>
- new package; version 5.17.2
- added patch gtest.patch
